# generated by datamodel-codegen:
#   filename:  openapi.json

from __future__ import annotations

from enum import Enum
from typing import List, Literal, Optional, Union

from pydantic import BaseModel, Extra, Field


class MlProblemDefinitionBase(BaseModel):
    class Config:
        extra = Extra.allow

    retrainEvery: Optional[float] = None
    """
    If provided, the model will automatically be retrained if this many milliseconds have passed since the last run. Models will not be retrained more frequently than once per day, and this policy is only checked once per day, so more than this many milliseconds may actually pass before the model is retrained.
    """


class LabelDefinitionBase(BaseModel):
    class Config:
        extra = Extra.allow

    id: Optional[str] = None
    """
    UUID uniquely identifying this label.
    """
    name: str
    """
    The unique name of the label.
    """
    description: str


class LabelDefinition(LabelDefinitionBase):
    id: str
    index: int = Field(..., ge=0, le=255)
    """
    The unique integer identifying this label, in the range [0,255]. It must never change during the life of the model because it is stored in training data as the label's identifier.
    """


class LabelsDefinitionInput(BaseModel):
    class Config:
        extra = Extra.allow

    labels: List[LabelDefinitionBase]


class LabelsDefinition(BaseModel):
    class Config:
        extra = Extra.forbid

    labels: List[LabelDefinition]
    maxLabelIndex: int = Field(..., ge=-1, le=255)
    """
    The maximum index used by any label defined for this model over the life of the model. This field cannot be set by a user but is used to track what label indices have already been exhausted.
    """


class ClassificationProblemInput(MlProblemDefinitionBase):
    labelDefinition: LabelsDefinitionInput


class ClassificationProblem(MlProblemDefinitionBase):
    labelDefinition: LabelsDefinition


class ImageSegmentationArea(BaseModel):
    """
    A run-length encoded (RLE) 4 channel color image, representing the mask for a single label.
    """

    class Config:
        extra = Extra.forbid

    id: str
    """
    The ID of the label this mask is for.
    """
    rle: List[int]
    """
    The run-length encoded mask; an array of 8-bit unsigned integers.
    """


class ImageSegmentationLabelData(BaseModel):
    """
    A raw image segmentation, in the format LabelStudio provides.
    """

    class Config:
        extra = Extra.forbid

    labelType: Literal["imgSeg"]
    projectId: str
    """
    The ID of the LifeOmic project the label file is saved under.
    """
    height: int
    """
    The height of the segmentation mask in pixels.
    """
    width: int
    """
    The width of the segmentation mask in pixels.
    """
    areas: List[ImageSegmentationArea]


class LabelFileData(BaseModel):
    __root__: ImageSegmentationLabelData


class Tag(BaseModel):
    class Config:
        extra = Extra.forbid

    name: str
    value: str


class Tags(BaseModel):
    __root__: List[Tag]


class LabelBase(BaseModel):
    class Config:
        extra = Extra.allow

    isConfirmed: Optional[bool] = None
    """
    A confirmed label is believed to be correct and may be used during training and evaluation.
    """
    lastConfirmedBy: Optional[str] = None
    """
    The email address or other identifier of the user who last confirmed this label.
    """
    updatedAt: Optional[float] = None
    """
    Timestamp for when the label was last updated or created expressed as milliseconds since the UTC epoch.
    """
    tags: Optional[Tags] = None


class Mask(BaseModel):
    """
    The fileId of an image containing per-pixel labels
    """

    class Config:
        extra = Extra.forbid

    fileId: str


class ImageSegmentationLabel(LabelBase):
    labelType: Literal["imgSeg"]
    mask: Mask
    """
    The fileId of an image containing per-pixel labels
    """


class ImageClassificationLabel(LabelBase):
    labelType: Literal["imgClf"]
    classes: List[float]
    """
    An array of integers representing the class(es) of the image
    """


class Label(BaseModel):
    __root__: Union[ImageSegmentationLabel, ImageClassificationLabel]


class ExampleBase(BaseModel):
    class Config:
        extra = Extra.allow

    id: str
    updatedAt: float
    """
    Timestamp expressed as milliseconds since the UTC epoch.
    """


class Image(BaseModel):
    class Config:
        extra = Extra.forbid

    fileId: str


class ImageSegmentationExample(ExampleBase):
    exampleType: Literal["imgSeg"]
    image: Image
    label: Optional[ImageSegmentationLabel] = None


class Image1(BaseModel):
    class Config:
        extra = Extra.forbid

    fileId: str


class ImageClassificationExample(ExampleBase):
    exampleType: Literal["imgClf"]
    image: Image1
    label: Optional[ImageClassificationLabel] = None


class Example(BaseModel):
    __root__: Union[ImageSegmentationExample, ImageClassificationExample]


class FhirCodesFilter(BaseModel):
    """
    Used to find FHIR resources containing a type code that equals any value in the codes array
    """

    class Config:
        extra = Extra.forbid

    filterType: Literal["FhirCodesFilter"]
    codes: List[str] = Field(..., max_items=10, min_items=1)


class CategoricalParameterSpace(BaseModel):
    class Config:
        extra = Extra.forbid

    type: Literal["categorical"]
    name: str
    values: List[str]


class NumericScale(Enum):
    auto = "auto"
    linear = "linear"
    log = "log"
    reverseLog = "reverseLog"


class NumericParameterSpace(BaseModel):
    class Config:
        extra = Extra.allow

    name: str
    min: float
    max: float
    scale: NumericScale


class ContinuousParameterSpace(NumericParameterSpace):
    type: Literal["continuous"]


class IntegerParameterSpace(NumericParameterSpace):
    type: Literal["integer"]


class ParameterSpace(BaseModel):
    __root__: Union[
        CategoricalParameterSpace, ContinuousParameterSpace, IntegerParameterSpace
    ]


class OptimizationObjective(BaseModel):
    class Config:
        extra = Extra.forbid

    direction: Literal["minimize", "maximize"]
    metric: str


class MetricDefinition(BaseModel):
    """
    Camel-cased mirror of https://docs.aws.amazon.com/sagemaker/latest/APIReference/API_MetricDefinition.html.
    """

    class Config:
        extra = Extra.forbid

    name: str
    regex: str


class TuningJobTrainingApproach(BaseModel):
    class Config:
        extra = Extra.allow

    type: Literal["tuningJob"]
    trainingImage: str = Field(
        ...,
        regex="^[0-9]+\\.dkr.ecr.[-a-z0-9]+\\.amazonaws\\.com\\/[-_a-zA-Z0-9]+:[-_a-zA-Z0-9]+$",
    )
    """
    An aws ecr image uri of the form <account-id>.dkr.ecr.<region>.amazonaws.com/<repo-name>:<tag>
    """
    metricDefinitions: List[MetricDefinition]
    objective: OptimizationObjective
    """
    The metric to optimize during the hyperparameter tuning process.
    """
    searchSpace: List[ParameterSpace]
    """
    The hyperparameter search space to explore during hyperparameter tuning.
    """
    maxTrials: Optional[int] = Field(None, ge=1, le=30)
    """
    The maximum number of candidate models to consider in a given model run.
    """


class TrainingApproach(BaseModel):
    __root__: TuningJobTrainingApproach


class DeployApproachBase(BaseModel):
    class Config:
        extra = Extra.allow

    inferenceImage: str = Field(
        ...,
        regex="^[0-9]+\\.dkr.ecr.[-a-z0-9]+\\.amazonaws\\.com\\/[-_a-zA-Z0-9]+:[-_a-zA-Z0-9]+$",
    )
    """
    An aws ecr image uri of the form <account-id>.dkr.ecr.<region>.amazonaws.com/<repo-name>:<tag>
    """


class EdgeDeployApproach(DeployApproachBase):
    type: Literal["edge"]


class CloudDeployApproach(DeployApproachBase):
    type: Literal["cloud"]


class DeployApproach(BaseModel):
    __root__: Union[EdgeDeployApproach, CloudDeployApproach]


class ModelConfigBase(BaseModel):
    class Config:
        extra = Extra.allow

    name: str
    description: str
    trainingApproach: TrainingApproach
    deployApproach: DeployApproach


class Split(BaseModel):
    class Config:
        extra = Extra.forbid

    n: float
    """
    The number of examples in this split.
    """
    start: float
    """
    Timestamp marking the beginning of the data for this split (inclusive). Expressed as milliseconds since the UTC epoch.
    """
    end: float
    """
    Timestamp marking the end of the data for this split (exclusive). Expressed as milliseconds since the UTC epoch.
    """
    uri: str = Field(..., regex="^s3:\\/\\/.+$")
    """
    S3 URI of where a copy of this exact split is saved.
    """


class Metric(BaseModel):
    class Config:
        extra = Extra.forbid

    name: str
    """
    The name of the metric e.g. "Cross Entopy Loss", "Accuracy", "F1 Macro", etc.
    """
    value: float
    stage: Literal["training", "evaluation"]
    """
    The stage of the model run this metric was computed in. If `training`, the metric could have been computed over the train or val set. If `evaluation`, the metric was computed over the test set.
    """


class ApprovalChoice(Enum):
    approved = "approved"
    rejected = "rejected"


class ApprovalDecisionBase(BaseModel):
    class Config:
        extra = Extra.forbid

    timestamp: float
    """
    Timestamp of when the decision was made. Expressed as milliseconds since the UTC epoch.
    """
    decision: ApprovalChoice


class SystemApprovalDecision(ApprovalDecisionBase):
    actor: Literal["system"]


class UserApprovalDecision(ApprovalDecisionBase):
    actor: Literal["user"]
    user: str


class ApprovalDecision(BaseModel):
    __root__: Union[SystemApprovalDecision, UserApprovalDecision]


class RunSplits(BaseModel):
    """
    Information about the different splits of the dataset used to train this model version.
    """

    class Config:
        extra = Extra.forbid

    train: Split
    val: Split
    test: Split


class RunMetrics(BaseModel):
    class Config:
        extra = Extra.forbid

    challenger: List[Metric]
    """
    Metrics about how the model version trained in this run (the challenger) performed on this run's test set.
    """
    champion: List[Metric]
    """
    Metrics about how the current best model (the champion) performed on this run's test set.
    """


class Parameter(BaseModel):
    """
    A hyperparameter value. The values themselves are represented as strings.
    """

    class Config:
        extra = Extra.forbid

    name: str
    value: str


class ImageSegmentationProblemBase(BaseModel):
    class Config:
        extra = Extra.allow

    problemType: Literal["imgSeg"]
    trainingDataFilter: FhirCodesFilter
    """
    Used to filter through patient data to identify image/mask pairs for model training.
    """


class ImageSegmentationProblemInput(
    ClassificationProblemInput, ImageSegmentationProblemBase
):
    pass


class ImageSegmentationProblem(ClassificationProblem, ImageSegmentationProblemBase):
    pass


class ImageClassificationProblemBase(BaseModel):
    class Config:
        extra = Extra.allow

    problemType: Literal["imgClf"]
    trainingDataFilter: FhirCodesFilter
    """
    Used to filter through patient data to identify the input images for the model.
    """


class ImageClassificationProblemInput(
    ClassificationProblemInput, ImageClassificationProblemBase
):
    pass


class ImageClassificationProblem(ClassificationProblem, ImageClassificationProblemBase):
    pass


class MlProblemDefinitionInput(BaseModel):
    __root__: Union[ImageSegmentationProblemInput, ImageClassificationProblemInput]


class MlProblemDefinition(BaseModel):
    __root__: Union[ImageSegmentationProblem, ImageClassificationProblem]


class ModelConfigInput(ModelConfigBase):
    problemDefinition: MlProblemDefinitionInput


class ModelConfig(ModelConfigBase):
    id: str
    """
    UUID uniquely identifying this model config.
    """
    accountId: str
    problemDefinition: MlProblemDefinition
    deployedId: Optional[str] = None
    """
    The ID of the model version currently deployed for this model config.
    """


class ModelRun(BaseModel):
    class Config:
        extra = Extra.forbid

    id: str
    """
    UUID uniquely identifying this model run.
    """
    slug: str
    """
    Human-readable slug that acts as an auto-generated name for the run. This field is not guaranteed to be unique, so cannot be used reliably as a unique identifier.
    """
    modelId: str
    """
    ID of the model config this run was produced for.
    """
    accountId: str
    modelArtifactUri: Optional[str] = None
    """
    S3 URI location where the final trained model artifact produced by this run is saved to.
    """
    status: Literal["running", "succeeded", "failed"]
    error: Optional[str] = None
    """
    If `status` is 'failed', gives the reason why the run failed.
    """
    deployStatus: Optional[Literal["deploying", "succeeded", "failed"]] = None
    deployError: Optional[str] = None
    """
    If `deployStatus` is 'failed', gives the reason why deploying the model endpoint failed
    """
    start: float
    """
    Timestamp of when the run started. Expressed as milliseconds since the UTC epoch.
    """
    end: Optional[float] = None
    """
    Timestamp of when the run ended. Expressed as milliseconds since the UTC epoch.
    """
    isArchived: Optional[bool] = None
    """
    True if this run has been archived. Archived runs' related artifacts are deleted, and they can no longer be deployed.
    """
    splits: Optional[RunSplits] = None
    hyperparameters: List[Parameter]
    """
    The hyperparameters used to train the model version created by this run.
    """
    metrics: RunMetrics
    approvals: List[ApprovalDecision]
    """
    Decisions made by various actors representing whether they think this model version should be used in production and become the new champion.
    """
    problemDefinition: MlProblemDefinition
    trainingApproach: TrainingApproach
    deployApproach: DeployApproach
